MarketplaceToCurrency = {
    "default": "USD",
    "EBAY_US": "USD",
    "EBAY_MOTORS_US": "USD",
    "EBAY_CA": "CAD",
    "EBAY_GB": "GBP",
    "EBAY_AU": "AUD",
    "EBAY_AT": "EUR",
    "EBAY_BE_FR": "EUR",
    "EBAY_FR": "EUR",
    "EBAY_DE": "EUR",
    "EBAY_IT": "EUR",
    "EBAY_BE_NL": "EUR",
    "EBAY_NL": "EUR",
    "EBAY_ES": "EUR",
    "EBAY_CH": "EUR",
    "EBAY_TW": "TWD",
    "EBAY_CZ": "EUR",
    "EBAY_DK": "EUR",
    "EBAY_FI": "EUR",
    "EBAY_GR": "EUR",
    "EBAY_HK": "HKD",
    "EBAY_HU": "EUR",
    "EBAY_IN": "INR",
    "EBAY_ID": "IDR",
    "EBAY_IE": "EUR",
    "EBAY_IL": "ILS",
    "EBAY_MY": "MYR",
    "EBAY_NZ": "NZD",
    "EBAY_NO": "EUR",
    "EBAY_PH": "PHP",
    "EBAY_PL": "EUR",
    "EBAY_PT": "EUR",
    "EBAY_PR": "USD",
    "EBAY_RU": "RUB",
    "EBAY_SG": "SGD",
    "EBAY_ZA": "ZAR",
    "EBAY_SE": "EUR",
    "EBAY_TH": "THB",
    "EBAY_VN": "VND",
    "EBAY_CN": "CNY",
    "EBAY_PE": "PEN",
    "EBAY_CA_FR": "CAD",
    "EBAY_JP": "JPY",
}
